package cn.com.qmth.examcloud.task.api.controller;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.task.base.bean.ReportsComputeBean;
import cn.com.qmth.examcloud.task.dao.entity.ReportsComputeEntity;
import cn.com.qmth.examcloud.task.service.ReportsComputeService;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

@RestController
@RequestMapping("${$rmp.ctr.task}" + "reportsCompute")
public class ReportsComputeController extends ControllerSupport {


	@Autowired
	ReportsComputeService reportsComputeService;

	@ApiOperation(value = "终止计算任务")
	@PostMapping("/stopJob/{id}")
	public void stopJob(@PathVariable @ApiParam(value = "ID") Long id){
		if (id == null) {
			throw new StatusException("1000001", "计算任务id不能为空");
		}
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();
		
		ReportsComputeEntity re=reportsComputeService.findById(id);
		
		if (re == null) {
			throw new StatusException("1000002", "计算任务不存在");
		}
		
		if(!rootOrgId.equals(re.getRootOrgId())) {
			throw new StatusException("1000003", "非法操作");
		}
		int ret=reportsComputeService.updateToStoping(re);
		if(ret==0) {
			throw new StatusException("1000004", "只能终止待处理和处理中的任务");
		}
	}
    @GetMapping("page/{projectId}/{pageNo}/{pageSize}")
    @ApiOperation(value = "计算任务列表分页查询")
    public PageInfo<ReportsComputeBean> queryPage(@PathVariable @ApiParam(value = "项目ID") Long projectId,
                                        @PathVariable @ApiParam(value = "pageNo = 1,2,3...") Integer pageNo, @PathVariable Integer pageSize) {
    	User user=getAccessUser();
    	return reportsComputeService.queryPage(projectId, pageNo, pageSize,user.getRootOrgId());
    }
    @ApiOperation(value="批量获取")
	@GetMapping("/getList")
	public List<ReportsComputeBean> getList(@RequestParam(required = true) String ids) {
    	List<Long> pids = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
    	User user = getAccessUser();
		return reportsComputeService.getList(pids, user.getRootOrgId());
	}
}
