package cn.com.qmth.examcloud.task.service.impl;

import java.lang.reflect.Method;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.RegExpUtil;
import cn.com.qmth.examcloud.global.api.HandleSyncCloudService;
import cn.com.qmth.examcloud.global.api.request.SyncBaseRequest;
import cn.com.qmth.examcloud.task.dao.DataSyncRepo;
import cn.com.qmth.examcloud.task.dao.entity.DataSyncEntity;
import cn.com.qmth.examcloud.task.service.DataSyncService;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年9月19日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class DataSyncServiceImpl implements DataSyncService {

	@Autowired
	DataSyncRepo dataSyncRepo;

	@Autowired
	HandleSyncCloudService handleSyncCloudService;

	@Override
	public void sync(String methodName, SyncBaseRequest req, Boolean async, Boolean retry) {
		String group = PropertyHolder.getString("$sync." + methodName + ".group");
		if (StringUtils.isBlank(group)) {
			throw new StatusException("001001", "group is not configured");
		}

		List<String> componentList = RegExpUtil.findAll(group, "[^\\,]+");

		for (String component : componentList) {
			component = component.trim();
			sync(component, methodName, req, async, retry);
		}
	}

	@Override
	public void sync(String component, String methodName, SyncBaseRequest req, Boolean async,
			Boolean retry) {
		String url = PropertyHolder.getString("$sync." + methodName + ".component." + component);
		req.setUrl(url);

		if (async) {
			DataSyncEntity entity = new DataSyncEntity();
			entity.setComponent(component);
			entity.setMethodName(methodName);
			entity.setParamType(req.getClass().getName());
			entity.setSyncNum(0);
			entity.setParamJson(JsonUtil.toJson(req));
			dataSyncRepo.saveAndFlush(entity);
		} else {
			try {
				Method method = handleSyncCloudService.getClass().getMethod(methodName,
						req.getClass());
				method.invoke(handleSyncCloudService, req);
			} catch (Exception e) {
				if (retry) {
					DataSyncEntity entity = new DataSyncEntity();
					entity.setComponent(component);
					entity.setMethodName(methodName);
					entity.setParamType(req.getClass().getName());
					entity.setSyncNum(0);
					entity.setParamJson(JsonUtil.toJson(req));
					dataSyncRepo.saveAndFlush(entity);
				} else {
					throw new RuntimeException(e);
				}
			}
		}

	}

}
