package cn.com.qmth.examcloud.task.service.job;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import cn.com.qmth.examcloud.api.commons.enums.NoticeReceiverRuleType;
import cn.com.qmth.examcloud.api.commons.enums.NoticeStatus;
import cn.com.qmth.examcloud.examwork.api.NoticeCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.NoticeRulePublishProgressBean;
import cn.com.qmth.examcloud.examwork.api.request.DisposePublishingUserNoticeReq;
import cn.com.qmth.examcloud.examwork.api.request.UpdateNoticeStatusReq;
import cn.com.qmth.examcloud.examwork.api.response.DisposePublishingUserNoticeResp;
import cn.com.qmth.examcloud.examwork.api.response.GetNoticeRulePublishProgressListResp;
import cn.com.qmth.examcloud.web.task.AbstractTask;
import cn.com.qmth.examcloud.web.task.ScheduleJob;
import cn.com.qmth.examcloud.web.task.TaskTracker;

/**
 * @author lideyin
 * 处理发布中的通知
 */
@Component("disposePublishingNoticeTask")
public class DisposePublishingNoticeTask extends AbstractTask {

    @Autowired
    TaskTracker taskTracker;

    @Autowired
    private NoticeCloudService noticeCloudService;

    @Override
    public void run(ScheduleJob scheduleJob) throws Exception {
        //获取待处理的通知进度数据
        GetNoticeRulePublishProgressListResp progressResponse = noticeCloudService.getToBeDisposedNoticeRulePublishProgressList();
        List<NoticeRulePublishProgressBean> progressList = progressResponse.getList();
        if (progressList == null || progressList.isEmpty()) {
            return;
        }


        for (NoticeRulePublishProgressBean progress : progressList) {
            Long startUserId = 0L;
            int loopTimes = 0;
            Long lastMaxUserId = getLastMaxUserId(progress.getNoticeReceiverRuleType(), progress);
            if (lastMaxUserId != null) {
                startUserId = lastMaxUserId + 1;
            }
            while (true) {
                DisposePublishingUserNoticeReq disposeReq = new DisposePublishingUserNoticeReq();
                disposeReq.setStartUserId(startUserId);
                disposeReq.setNoticeRulePublishProgress(progress);
                DisposePublishingUserNoticeResp disposeResp = noticeCloudService.disposePublishingUserNotice(disposeReq);
                Long nextUserId = disposeResp.getNextUserId();

                if (nextUserId.equals(startUserId)) {
                    updateNoticeStatus(progress.getNoticeId(), NoticeStatus.PUBLISHED);
                    break;
                } else {
                    startUserId = nextUserId;
                    //处理中的状态只需更新一次
                    if (loopTimes == 1) {
                        updateNoticeStatus(progress.getNoticeId(), NoticeStatus.PUBLISHING);
                    }
                }
                loopTimes++;
            }
        }


    }

    @Override
    public TaskTracker getTaskTracker() {
        return taskTracker;
    }

    private void updateNoticeStatus(Long noticeId, NoticeStatus noticeStatus) {
        UpdateNoticeStatusReq updateNoticeStatusReq = new UpdateNoticeStatusReq();
        updateNoticeStatusReq.setNoticeId(noticeId);
        updateNoticeStatusReq.setNoticeStatus(noticeStatus);
        noticeCloudService.updateNoticeStatus(updateNoticeStatusReq);
    }

    /**
     * 获取上次更新的最大用户id
     *
     * @param ruleType
     * @param process
     * @return
     */
    private Long getLastMaxUserId(NoticeReceiverRuleType ruleType,
                                  NoticeRulePublishProgressBean process) {
        if (ruleType == NoticeReceiverRuleType.ALL_STUDENTS_OF_ROOT_ORG
                || ruleType == NoticeReceiverRuleType.STUDENTS_OF_EXAM) {
            return process.getMaxStudentId();
        } else {
            return process.getMaxCommonUserId();
        }
    }
}
