package cn.com.qmth.examcloud.task.service.producer;

import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.examcloud.core.oe.admin.api.ExamRecordForMarkingCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordForMarkingBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.QueryValidExamRecordInfoPageReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.QueryValidExamRecordInfoPageResp;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamCourseRelationBean;
import cn.com.qmth.examcloud.examwork.api.request.GetExamCourseListReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamCourseListResp;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.task.base.multithread.Producer;
import cn.com.qmth.examcloud.task.service.dto.MarkWorkCreateDto;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;

public class MarkWorkCreateProducer extends Producer {

    private ExamRecordForMarkingCloudService examRecordForMarkingCloudService = SpringContextHolder
            .getBean(ExamRecordForMarkingCloudService.class);

    private ExamCloudService examCloudService = SpringContextHolder.getBean(ExamCloudService.class);

    private static final Logger logger = LoggerFactory.getLogger(MarkWorkCreateProducer.class);

    @Override
    protected void produce(Map<String, Object> param) throws Exception {
        Long examId = (Long) param.get("examId");
        Long workId = (Long) param.get("workId");
        // 根据考试批次Id去考务中获取课程信息集合
        GetExamCourseListReq getExamCourseListReq = new GetExamCourseListReq();
        getExamCourseListReq.setExamId(examId);
        Long batchSize = 100L;
        SysPropertyCacheBean spc = CacheHelper.getSysProperty("marking.rpc.savePaperList.batchSize");
        if (spc != null && spc.getValue() != null) {
            batchSize = (Long) spc.getValue();
        }
        Long start = 1l;
        while (true) {// 每次去考务调用100条记录（目前暂无处理没有考生的课程）
            getExamCourseListReq.setStart(start);
            GetExamCourseListResp getExamCourseListResp = examCloudService.getExamCourseList(getExamCourseListReq);

            for (ExamCourseRelationBean course : getExamCourseListResp.getRelationList()) {
                createStudentPapers(examId, workId, course, batchSize);
            }
            if (start == getExamCourseListResp.getNext()) {
                break;
            }
            start = getExamCourseListResp.getNext();
        }
    }

    private void createStudentPapers(Long examId, Long workId, ExamCourseRelationBean examCourseRelationBean,
            Long batchSize) throws InterruptedException {
        int count = 0;
        logger.info("创建评卷工作课程答卷：" + examCourseRelationBean.getCourseCode() + examCourseRelationBean.getCourseName());
        // 通过“考试批次”和“课程Code”获取需要阅卷的考卷信息集合
        QueryValidExamRecordInfoPageReq req = new QueryValidExamRecordInfoPageReq();
        req.setBatchNum(workId + "");
        req.setExamId(examId);
        req.setCourseId(examCourseRelationBean.getCourseId());
        req.setSize(batchSize);
        Long start = 1l;
        while (true) {
            req.setStart(start);
            QueryValidExamRecordInfoPageResp resp = examRecordForMarkingCloudService.queryValidExamRecordInfoPage(req);
            count = resp.getExamRecordForMarkingBeanList().size();
            if (count > 0) {
                subListDispose(resp.getExamRecordForMarkingBeanList(), workId, examCourseRelationBean, batchSize);
            }
            if (start.equals(resp.getNext())) {
                break;
            }
            start = resp.getNext();
        }
    }

    private void subListDispose(List<ExamRecordForMarkingBean> list, Long workId,
            ExamCourseRelationBean examCourseRelationBean, Long batchSize) throws InterruptedException {
        if (list.size() <= batchSize) {
            // 生产需处理的数据
            offer(new MarkWorkCreateDto(list, workId, examCourseRelationBean));
        } else {
            int size = list.size();
            int len = batchSize.intValue();
            int count = (size + len - 1) / len;

            for (int i = 0; i < count; i++) {
                List<ExamRecordForMarkingBean> subList = list.subList(i * len,
                        ((i + 1) * len > size ? size : len * (i + 1)));
                // 生产需处理的数据
                offer(new MarkWorkCreateDto(subList, workId, examCourseRelationBean));
            }
        }
    }
}
